SystemOrganization addCategory: #SqueakShell!

Project subclass: #SqshProject
	instanceVariableNames: 'uiProcess lastCycleTime lastCycleDuration lastCycleDurationUpdate interCycleDelay canvas lastKeyboardEvent lastMouseEvent controllers activeController keyboardInterpreter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshProject methodsFor: 'scheduling' stamp: 'mt 2/15/2016 18:13'!
addDeferredUIMessage: valuableObject 

	self world activeController deferredMessages nextPut: valuableObject.! !

!SqshProject methodsFor: 'utilities' stamp: 'mt 2/14/2016 14:34'!
addItem: item toMenu: menu selection: action requestor: requestingProject

	requestingProject
		addItem: item
		toMenu: menu
		selection: action
		color: Color white
		thumbnail: thumbnail.! !

!SqshProject methodsFor: 'initialization' stamp: 'mt 2/15/2016 16:59'!
defaultBackgroundColor

	^ Color white! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 2/15/2016 18:48'!
displaySizeChanged

	DisplayScreen startUp. self flag: #move. "mt: Move to Project?"
	self resetDisplay.! !

!SqshProject methodsFor: 'initialization' stamp: 'mt 2/15/2016 17:57'!
initialize

	super initialize.

	world := SqshControllerManager new.! !

!SqshProject methodsFor: 'scheduling' stamp: 'mt 2/15/2016 16:03'!
interruptName: labelString
	"Create a Notifier on the active scheduling process with the given label."

	^ self interruptName: labelString preemptedProcess: nil! !

!SqshProject methodsFor: 'scheduling' stamp: 'mt 2/15/2016 16:13'!
interruptName: labelString preemptedProcess: theInterruptedProcess
	"Create a Notifier on the active scheduling process with the given label."

	| preemptedProcess projectProcess |
	projectProcess := self uiProcess.
	
	preemptedProcess := theInterruptedProcess ifNil: [Processor preemptedProcess].
	"Only debug preempted process if its priority is >= projectProcess' priority"
	
	preemptedProcess priority < projectProcess priority 
		ifTrue:[preemptedProcess := projectProcess].
	
	preemptedProcess suspend.
	ToolSet interrupt: preemptedProcess label: labelString.! !

!SqshProject methodsFor: 'testing' stamp: 'mt 2/12/2016 14:35'!
isMorphic

	^ false! !

!SqshProject methodsFor: 'enter' stamp: 'mt 2/12/2016 15:01'!
pauseSoundPlayers
	"Pause sound players, subject to preference settings"

	Smalltalk at: #ScorePlayer ifPresent:[:playerClass| 
		playerClass allSubInstancesDo: [:player | player pause]]! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 2/15/2016 18:00'!
resetDisplay

	self world resetDisplay.! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 2/15/2016 18:02'!
restore
	"Used for zoom effect. Use the current display canvas, which is not the one that the world is holding."

	self world displayNowOn: Display getCanvas.! !

!SqshProject methodsFor: 'displaying' stamp: 'mt 2/15/2016 18:45'!
restoreDisplay

	self resetDisplay.! !

!SqshProject methodsFor: 'enter' stamp: 'mt 2/15/2016 18:02'!
saveState

	changeSet := ChangeSet current.
	thumbnail ifNotNil: [:form | form hibernate].
	transcript := Transcript.

	transcript removeDependent: self world activeController.! !

!SqshProject methodsFor: 'enter' stamp: 'mt 2/15/2016 18:18'!
scheduleProcessForEnter: showZoomAnimation

	self spawnNewProcess.
	Processor terminateActive.! !

!SqshProject methodsFor: 'dispatching' stamp: 'mt 2/15/2016 15:17'!
selectorPrefixForDispatch

	^ 'sqsh'! !

!SqshProject methodsFor: 'enter' stamp: 'mt 2/15/2016 18:07'!
setWorldForEnterFrom: old recorder: recorderOrNil

	World := nil. "Suspicious...*sigh*"
	
	transcript addDependent: self world activeController.! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/15/2016 18:05'!
spawnNewProcess

	uiProcess := [
		[
			DisplayScreen checkForNewScreenSize.			
			self world doOneCycle.			
			Processor yield.
		] repeat.
	] newProcess.

	uiProcess
		priority: Processor userSchedulingPriority;
		resume.! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/12/2016 16:28'!
spawnNewProcessIfThisIsUI: suspendedProcess
	"Initialize a UI process if needed. Answer true if suspendedProcess was interrupted
	from a UI process."

	self uiProcess == suspendedProcess ifTrue: [
		self spawnNewProcess.
		^true
	].
	^false		"no new process was created"
! !

!SqshProject methodsFor: 'accessing' stamp: 'mt 2/12/2016 15:13'!
subProjects

	^ #()! !

!SqshProject methodsFor: 'active process' stamp: 'mt 2/12/2016 13:01'!
uiProcess

	^ uiProcess! !

!PluggableFileList methodsFor: '*SqueakShell-Support' stamp: 'mt 2/15/2016 20:11'!
sqshOpenLabel: aString in: aWorld

	Project current world activeController
		showMessage: self directory printString;
		showMessage: (self fileList joinSeparatedBy: String cr).! !

!MailComposition methodsFor: '*SqueakShell-Support' stamp: 'mt 2/15/2016 20:06'!
sqshOpen

	Project current world activeController
		showMessage: 'Sorry, there is no support for mail send in the Squeak Shell.'.! !

UIManager subclass: #SqshUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshUIManager class methodsFor: 'class initialization' stamp: 'mt 2/15/2016 18:15'!
isActiveManager

	^ Project current world class == SqshControllerManager! !

!SqshUIManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:22'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock
	
	^ Project current world activeController
		showProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock.! !

!SqshUIManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:22'!
inform: msg

	Project current world activeController showMessage: msg.! !

!SqshUIManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:22'!
informUserDuring: block

	Project current world activeController showMessageDuring: block.! !

!SqshUIManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:52'!
restoreDisplay

	self flag: #why. "mt: Is this really necessary?"
	Project current restoreDisplay.! !

!SyntaxError class methodsFor: '*SqueakShell-Support' stamp: 'mt 2/15/2016 20:04'!
sqshOpen: aSyntaxError

	| process |
	process := Processor activeProcess.
	
	Project current spawnNewProcessIfThisIsUI: process.

	Project current addDeferredUIMessage: [
		Project current world activeController
			showMessage: aSyntaxError contents.
		process terminate.
	].
			
	process suspend.! !

Object subclass: #SqshController
	instanceVariableNames: 'model view deferredMessages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/14/2016 13:42'!
bindingOf: aString

	^ self model bindingOf: aString! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:56'!
deferredMessages

	^ deferredMessages! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:56'!
deferredMessages: anObject

	deferredMessages := anObject! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/15/2016 18:14'!
evaluate: string

	| result code newReceiver |
	string ifEmpty: [^ self].
	
	code := string.
	newReceiver := false.

	(code beginsWith: '!!') ifTrue: [
		code = '!!s' ifTrue: [^ Smalltalk snapshot: true andQuit: false].
		code = '!!q' ifTrue: [^ Smalltalk snapshot: true andQuit: true].
		code = '!!l' ifTrue: [^ self changed: #leave].
		code = '!!clear' ifTrue: [^ self model clearLines].
		code = '!!help' ifTrue: [^ self showHelp].
		code = '!!!!' ifTrue: [^ self model usePreviousReceiver].
		code = '!!logo' ifTrue: [^ self showMessage: SqueakShell logo].
		code = '!!bindings' ifTrue: [^ self showBindings].
		code = '!!update' ifTrue: [^ Utilities updateFromServer].
	
		newReceiver := true.
		code := code allButFirst].
	
	[result := Compiler new
		evaluate: code
		in: self model context
		to: self model receiver
		notifying: self
		ifFail: [^ self]
		logged: true.
	] on: ParserNotification do: [:ex |
		self showMessage: ex printString.
		^ self].

	newReceiver
		ifTrue: [self model changeReceiver: result]
		ifFalse: [self showMessage: result printString].! !

!SqshController methodsFor: 'events' stamp: 'mt 2/14/2016 18:02'!
handleKeyboard: keyCharacter modifiers: someModifiers

	self view autoScroll: true.

	"Evaluate the input."
	(keyCharacter = Character enter or: [keyCharacter = Character cr])
		ifTrue: [
			someModifiers first "shift"
				ifFalse: [^ self model acceptInput]].
	
	"Text correction."
	keyCharacter = Character backspace
		ifTrue: [^ self model backspace; moveCursorLeft].
	keyCharacter = Character delete
		ifTrue: [^ self model forwardDelete].
	(keyCharacter = $v and: [someModifiers third "CMD" | someModifiers second "CTRL"])
		ifTrue: [^ self model addInput: Clipboard clipboardText asString].
	
	"Text cursor movement."
	keyCharacter = Character arrowLeft
		ifTrue: [^ self model moveCursorLeft].
	keyCharacter = Character arrowRight
		ifTrue: [^ self model moveCursorRight].

	"Command history."
	keyCharacter = Character arrowUp
		ifTrue: [^ self model showPreviousCommand].
	keyCharacter = Character arrowDown
		ifTrue: [^ self model showNextCommand].
	
	"Regular character input."
	self model addInput: keyCharacter asString.
	self model moveCursorRight.! !

!SqshController methodsFor: 'events' stamp: 'mt 2/14/2016 16:55'!
handleMouseWheel: upOrDown

	upOrDown = #up
		ifTrue: [self view scrollUp: 5 "lines"].
	upOrDown = #down
		ifTrue: [self view scrollDown: 5 "lines"].! !

!SqshController methodsFor: 'initialize-release' stamp: 'mt 2/15/2016 17:09'!
initialize

	super initialize.

	self
		deferredMessages: SharedQueue new;
		model: SqshModel new;
		view: SqshView new.
			
	"Give the view some information to display."
	self view
		autoScroll: true;
		promptVisible: true;
		defaultFont: Preferences standardCodeFont;
		model: self model.
		
	self showWelcomeMessage.! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
model

	^ model! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:10'!
model: anObject

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].

	model := anObject! !

!SqshController methodsFor: 'evaluation' stamp: 'mt 2/14/2016 13:48'!
notify: msg at: pos in: stream

	| source |
	source := stream contents.
	
	self showMessage: (source copyFrom: 1 to: pos-1), msg, (source copyFrom: pos to: source size).! !

!SqshController methodsFor: 'evaluation - interactive' stamp: 'mt 2/14/2016 14:18'!
select! !

!SqshController methodsFor: 'evaluation - interactive' stamp: 'mt 2/14/2016 14:18'!
selectFrom: b to: c
	"Ignore."! !

!SqshController methodsFor: 'evaluation - interactive' stamp: 'mt 2/14/2016 13:43'!
selectionInterval

	^ 1 to: 0! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/14/2016 21:54'!
showBindings

	| max |
	max := 0.
	self model bindings keys do: [:key | max := max max: key size].
	max := max + 3.
	
	self model bindings keysAndValuesDo: [:identifier :object |
		self showMessage: (identifier padded: #right to: max with: $.), (object printStringLimitedTo: 40)].! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/14/2016 18:40'!
showHelp

	self showMessage: '\The Squeak Shell\****************\\GENERAL\    !!s ... Save the image\    !!q ... Save the image and quit\    !!l ... Leave this project and enter the parent project if any\ !!help ... Show this help.\ !!logo ... Guess what?!!\\BINDINGS\         !!<code> ... Bind ''self'' to the result of the <code>\              !!!! ... Use previous binding\ <var> := <code> ... Bind <var> to the result of <code>\\EDITING AND HISTORY\             !!clear ... Clear screen and output history\           Arrow-Up ... Previous command\         Arrow-Down ... Next command\           Wheel-Up ... Scroll up and deactivate auto-scroll\         Wheel-Down ... Scroll down and activate auto-scroll\       Return/Enter ... Evaluate the command\ Shift+Return/Enter ... Insert line break\              CMD+V ... Paste from the clipboard' withCRs.! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/14/2016 21:56'!
showMessage: string

	self model addLine: string asString.! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/15/2016 15:00'!
showMessageDuring: workBlock

	^ workBlock value: [:info |
		self model addLine: info asString.
		self changed: #display]! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/15/2016 15:00'!
showProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock

	| result progress lineLabel lineProgress |
	lineLabel := self model addLine: (titleString ifEmpty: ['Please wait...']).
	lineProgress := self model addLine: ''.
	
	progress := [:barValue | | p |
		p := (barValue - minVal) / ((maxVal - minVal) max: 1).
		lineProgress contents: ('{1}% [{2}>{3}] {4}/{5}' format: {
			(p * 100) rounded asString padded: #left to: 3 with: Character space.
			String new: (p * 40) rounded withAll: $=.
			String new: (1-p * 40) rounded withAll: Character space.
			barValue.
			maxVal.}).
			
		self changed: #display].
	
	[result := workBlock value: progress]
		on: ProgressNotification
		do: [:ex |
			ex extraParam isString ifTrue: [lineLabel contents: ex extraParam].
			ex resume].

	progress value: maxVal.

	^ result! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/14/2016 14:05'!
showStack: methodContext
	"Debugging."
	
	self model addLines: ((methodContext stackOfSize: 20) collect: [:ctxt | ctxt printString]).! !

!SqshController methodsFor: 'printing' stamp: 'mt 2/15/2016 18:14'!
showWelcomeMessage

	self showMessage: SqueakShell logo.
	self showMessage: 'Welcome to the Squeak Shell!!\\We suggest to use a monospaced code font.\Type !!help to get some help.\If you are brave enough, try evaluating some Smalltalk code.\\' withCRs.! !

!SqshController methodsFor: 'evaluation - interactive' stamp: 'mt 2/14/2016 14:25'!
text

	^ ''! !

!SqshController methodsFor: 'updating' stamp: 'mt 2/15/2016 20:19'!
update: aspect

	super update: aspect.
	
	(aspect = #appendEntry or: [aspect = #appendEntryLater])
		ifTrue: [
			self showMessage: Transcript contents.

			"Force screen update."
			aspect = #appendEntry
				ifTrue: [self changed: #display]].! !

!SqshController methodsFor: 'updating' stamp: 'mt 2/15/2016 17:14'!
update: aspect with: object

	aspect = #accept ifTrue: [
		"Store the code to evaluate in the history."
		self model
			addLine: self model prompt;
			clearInput.
		
		"Hide prompte, execute code, show prompt again."
		self view promptVisible: false.
		[self evaluate: object]
			ensure: [self view promptVisible: true]].! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
view

	^ view! !

!SqshController methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
view: anObject

	view := anObject! !

Object subclass: #SqshControllerManager
	instanceVariableNames: 'lastCycleTime lastCycleDuration lastCycleDurationUpdate interCycleDelay canvas lastKeyboardEvent lastMouseEvent scheduledControllers activeController keyboardInterpreter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
activeController

	^ activeController! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
activeController: anObject

	activeController := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
canvas

	^ canvas! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
canvas: anObject

	canvas := anObject! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:01'!
displayNowOn: canvas
	"Enforce a redraw now."
	
	activeController view
		updateDisplay;
		displayOn: canvas.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:55'!
displayOn: canvas

	activeController view displayOn: canvas.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:05'!
doOneCycle
				
	self processDeferredMessages.
	self processInputEvents.
	self handleScreenUpdate.
	
	self interCyclePause.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:39'!
handleKeyboardEvent: event

	| shiftPressed controlKeyPressed commandKeyPressed keyCharacter |
	shiftPressed := (event fifth bitShift: 3) anyMask: 8.
	controlKeyPressed := (event fifth bitShift: 3) anyMask: 16.
	commandKeyPressed := (event fifth bitShift: 3) anyMask: 64.
	keyCharacter := event third asCharacter.
	
	"Quick escape back to parent project."
	keyCharacter = Character escape
		ifTrue: [Project returnToPreviousProject].

	(controlKeyPressed and: [keyCharacter = Character arrowUp])
		ifTrue: [^ activeController handleMouseWheel: #up].
	(controlKeyPressed and: [keyCharacter = Character arrowDown])
		ifTrue: [^ activeController handleMouseWheel: #down].		

	event fourth = 0 "key stroke"
		ifTrue: [activeController
			handleKeyboard: (keyboardInterpreter nextCharFrom: EventSensor default firstEvt: event)
			modifiers: {shiftPressed. controlKeyPressed. commandKeyPressed}].

	lastKeyboardEvent := event.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:56'!
handleMouseEvent: event

	lastMouseEvent := event.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:55'!
handleScreenUpdate

	Display deferUpdates: true.

	self displayOn: canvas.
		
	canvas finish.
			
	Display
		forceToScreen;
		deferUpdates: false;
		forceDisplayUpdate.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:32'!
initialize

	super initialize.

	keyboardInterpreter := LanguageEnvironment currentPlatform class defaultInputInterpreter.

	scheduledControllers := {SqshController new}.

	activeController := scheduledControllers first.
	activeController addDependent: self.
	
	self reset.! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
interCycleDelay

	^ interCycleDelay! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
interCycleDelay: anObject

	interCycleDelay := anObject! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:53'!
interCyclePause

	| millisecondsToWait minimumSpan |
	
	"Update FPS counter every second."
	(Time millisecondsSince: lastCycleDurationUpdate) > 1000
		ifTrue: [
			lastCycleDurationUpdate := Time millisecondClockValue.
			lastCycleDuration := lastCycleTime ifNil: [9999999] ifNotNil: [:t | Time millisecondsSince: t]].
	
	minimumSpan := (Preferences higherPerformance ifTrue: [0] ifFalse: [20]).
	
	millisecondsToWait := lastCycleTime
		ifNil: [0] ifNotNil: [:t | minimumSpan - (Time millisecondsSince: t) abs].
		
	(millisecondsToWait > 0 and: [millisecondsToWait <= minimumSpan]) ifTrue: [
		(interCycleDelay isNil or: [ interCycleDelay beingWaitedOn ])
			ifTrue: [ interCycleDelay := Delay forMilliseconds: millisecondsToWait ]
			ifFalse: [ interCycleDelay delayDuration: millisecondsToWait ].
		interCycleDelay wait ].
	
	lastCycleTime := Time millisecondClockValue.! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
keyboardInterpreter

	^ keyboardInterpreter! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
keyboardInterpreter: anObject

	keyboardInterpreter := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleDuration

	^ lastCycleDuration! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleDuration: anObject

	lastCycleDuration := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleDurationUpdate

	^ lastCycleDurationUpdate! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleDurationUpdate: anObject

	lastCycleDurationUpdate := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleTime

	^ lastCycleTime! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastCycleTime: anObject

	lastCycleTime := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastKeyboardEvent

	^ lastKeyboardEvent! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastKeyboardEvent: anObject

	lastKeyboardEvent := anObject! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastMouseEvent

	^ lastMouseEvent! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:51'!
lastMouseEvent: anObject

	lastMouseEvent := anObject! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:57'!
processDeferredMessages

	| object |
	[object := activeController deferredMessages nextOrNil] whileNotNil: [
		object value].! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 17:57'!
processInputEvents

	| event |
	[event := EventSensor default nextEvent] whileNotNil: [

		event first = 1 "mouse event"
			ifTrue: [self handleMouseEvent: event].
	
		event first = 2 "keyboard event"
			ifTrue: [self handleKeyboardEvent: event].
	
	].! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:09'!
reset

	canvas := Display getCanvas.
	lastCycleDuration := 9999999.
	lastCycleDurationUpdate := Time millisecondClockValue.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:01'!
resetDisplay
	"Bring the display to a usable state after handling primitiveError."

	self displayNowOn: canvas.! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:32'!
scheduledControllers

	^ #()! !

!SqshControllerManager methodsFor: 'accessing' stamp: 'mt 2/15/2016 18:32'!
scheduledControllers: anObject

	scheduledControllers := anObject! !

!SqshControllerManager methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:39'!
update: aspect

	super update: aspect.
	
	aspect = #leave ifTrue: [Project returnToPreviousProject].
	aspect = #display ifTrue: [self handleScreenUpdate].! !

Object subclass: #SqshModel
	instanceVariableNames: 'lines receivers context cursor bindings commands commandIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 11:40'!
acceptInput

	self changed: #accept with: self input.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:05'!
addInput: string

	self input: (self input copyFrom: 1 to: self cursor x), string, (self input copyFrom: self cursor x + 1 to: self input size).
	self changed: #input.	! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/15/2016 14:00'!
addLine: string

	| result |
	result := self lines add: (ValueHolder new contents: string).
	result addDependent: self.
	
	self changed: #lines.
	^ result! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/15/2016 13:49'!
addLines: someLines

	| result |
	result := self lines addAll: (someLines collect: [:line | ValueHolder new contents: line]).
	self changed: #lines.
	^ result! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:05'!
backspace

	self cursor x = 0 ifTrue: [^ self].

	self input: (self input copyFrom: 1 to: self cursor x - 1), (self input copyFrom: self cursor x + 1 to: self input size).
	self changed: #input.! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 13:34'!
bindingOf: aString 

	(self bindings includesKey: aString) ifFalse: [
		aString first isUppercase
			ifTrue: [^nil]
			ifFalse: [bindings at: aString put: nil]].
		
	^ bindings associationAt: aString! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:32'!
bindings

	^ bindings! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:32'!
bindings: anObject

	bindings := anObject! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 17:03'!
changeReceiver: newReceiver

	self receivers add: newReceiver.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:05'!
clearInput

	self input ifEmpty: [^ self].

	(self commandIndex < self commands size and: [self commands last notEmpty])
		ifTrue: [self commands add: self input]
		ifFalse: [self commands atLast: 1 put: self input].
		
	self commands add: ''.
	self commandIndex: self commands size.

	self cursor: 0 @ self cursor y.
	self changed: #input.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 14:17'!
clearLines

	self lines removeAll.
	self changed: #lines.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 15:30'!
commandIndex

	^ commandIndex! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:05'!
commandIndex: anObject

	commandIndex := anObject.
	self changed: #input.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 15:23'!
commands

	^ commands! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 15:25'!
commands: someCommands

	commands := someCommands.
	commandIndex := commands size.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:29'!
context

	^ context! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:29'!
context: anObject

	context := anObject! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:15'!
cursor

	^ cursor! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 13:15'!
cursor: anObject

	cursor := anObject! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:06'!
forwardDelete

	self cursor x >= self input size ifTrue: [^ self].
	
	self input: (self input copyFrom: 1 to: self cursor x), (self input copyFrom: self cursor x + 2 to: self input size).
	self changed: #input.! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 13:34'!
hasBindingOf: aString 

	^bindings includesKey: aString.! !

!SqshModel methodsFor: 'initialize-release' stamp: 'mt 2/14/2016 22:02'!
initialize

	super initialize.
	
	self
		lines: OrderedCollection new;
		receivers: OrderedCollection new;
		commands: {''} asOrderedCollection;
		cursor: 0@1;
		bindings: OrderedDictionary new.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 15:34'!
input

	^ self commands at: self commandIndex! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 15:55'!
input: anObject

	(self commandIndex < self commands size and: [self commands last notEmpty])
		ifTrue: [self commands add: anObject]
		ifFalse: [self commands atLast: 1 put: anObject].

	self commandIndex: self commands size! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:56'!
lines

	^ lines! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:06'!
lines: anObject

	lines := anObject! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:08'!
moveCursorLeft

	self cursor: (self cursor x - 1 max: 0) @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:08'!
moveCursorRight

	self cursor: (self cursor x + 1 min: self input size) @ self cursor y.
	self changed: #cursor.! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 12:32'!
prompt
	"Connection of receiver string and input."
	
	^ self receiverString, self input! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 17:01'!
receiver

	^ self receivers
		ifEmpty: [nil]
		ifNotEmpty: [:r | r last]! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 17:16'!
receiverString
	
	^ ((self receivers ifEmpty: [{nil}]) collect: [:object | '[', (object printStringLimitedTo: 10), ']'])
		join, '>'! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 17:00'!
receivers

	^ receivers! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 17:00'!
receivers: anObject

	receivers := anObject! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:06'!
sends

	^ sends! !

!SqshModel methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:06'!
sends: anObject

	sends := anObject! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:07'!
showNextCommand

	self commandIndex: (self commandIndex + 1 min: self commands size).
	self cursor: self input size @ self cursor y.
	self changed: #input.! !

!SqshModel methodsFor: 'editing' stamp: 'mt 2/14/2016 22:07'!
showPreviousCommand

	self commandIndex: (self commandIndex - 1 max: 1).
	self cursor: self input size @ self cursor y.
	self changed: #input.! !

!SqshModel methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 14:01'!
update: aspect

	"We give value holders to the outside so that controllers etc. can modify lines."
	aspect = #contents ifTrue: [self changed: #lines].! !

!SqshModel methodsFor: 'bindings' stamp: 'mt 2/14/2016 17:04'!
usePreviousReceiver

	self receivers ifEmpty: [^ self].
	self receivers removeLast.! !

Object subclass: #SqshView
	instanceVariableNames: 'model defaultFont needsDisplayUpdate scrollingOffset autoScroll promptVisible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:20'!
autoScroll

	^ autoScroll! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:40'!
autoScroll: boolean

	autoScroll := boolean.
	self updateDisplay.! !

!SqshView methodsFor: 'defaults' stamp: 'mt 2/14/2016 11:36'!
defaultBackgroundColor

	^ Color white! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:20'!
defaultFont
	^ defaultFont! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:20'!
defaultFont: aFont
	defaultFont := aFont.! !

!SqshView methodsFor: 'defaults' stamp: 'mt 2/14/2016 11:21'!
defaultLineHeight

	^ self defaultFont height! !

!SqshView methodsFor: 'defaults' stamp: 'mt 2/14/2016 11:36'!
defaultTextColor

	^ Color black! !

!SqshView methodsFor: 'defaults' stamp: 'mt 2/14/2016 13:11'!
defaultTextCursorColor

	^ Color blue! !

!SqshView methodsFor: 'display' stamp: 'mt 2/14/2016 16:25'!
displayLine: string at: pt on: canvas
	"Adds support for line breaks. Works best with monospaced fonts (for performance reasons). Returns resulting offset and intervals."

	| charactersPerLine position o i |
	"Empty line?"
	string ifEmpty: [^ {pt + (0@ self defaultLineHeight). {1 to: string size}}].
	
	charactersPerLine := self width // (self defaultFont widthOf: $x).
	position := 1.
	
	"Display intervals."
	i := OrderedCollection new.
	o := pt.	
	[position <= string size] whileTrue: [
		i add: (position to: (position + charactersPerLine - 1 min: string size)).
		canvas drawString: string from: i last first to: i last last at: o font: self defaultFont color: self defaultTextColor.
		o := o + (0@ self defaultLineHeight).
		position := position + charactersPerLine].
	
	^ {o. i}! !

!SqshView methodsFor: 'display' stamp: 'mt 2/15/2016 17:12'!
displayOn: canvas
	
	| offset |
	needsDisplayUpdate == true ifFalse: [^ self].
	
	canvas fillColor: self defaultBackgroundColor.
	
	"Avoid text layouting issues if we are too narrow."
	self width < self minimumWidth ifTrue: [^ self].
	
	offset := self autoScroll
		ifTrue: [self offsetForAutoScroll]
		ifFalse: [self scrollingOffset].
		
	"Show previous lines."
	self model lines do: [:lineWithBreaks |
		lineWithBreaks contents
			ifEmpty: [offset := (self displayLine: '' at: offset on: canvas) first]
			ifNotEmpty: [:line | line lineIndicesDo: [:start :end :endCr |
				offset := (self displayLine: (lineWithBreaks contents copyFrom: start to: end) at: offset on: canvas) first]]].

	"Show current prompt and cursor. Support a multi-line prompt."
	self promptVisible ifTrue: [
		self displayPrompt: self model prompt at: offset on: canvas].
		
	needsDisplayUpdate := false.! !

!SqshView methodsFor: 'display' stamp: 'mt 2/15/2016 17:07'!
displayPrompt: prompt at: pt on: canvas
	"Show current prompt and cursor. Support a multi-line prompt."
	
	| offset intervals vCursor |
	offset := pt.
	vCursor := self model receiverString size + self model cursor x.
	
	prompt lineIndicesDo: [:start :end :endCr |
		| tmp |
		tmp := self displayLine: (prompt copyFrom: start to: end) at: offset on: canvas.

		intervals := tmp second.
		(vCursor between: start and: endCr)
			ifTrue: [
				"Found the line with the text cursor in it!! Show it."
				intervals
					detect: [:i |  (i includes: vCursor-start+1) ifTrue: [true] ifFalse: [
						offset := offset + (0@ self defaultLineHeight).
						false]]
					ifFound: [:i | offset := (self defaultFont widthOfString: (prompt copyFrom: start to: end) from: i first to: vCursor-start+1) @ offset y]
					ifNone: [].
				canvas line: offset to: offset + (0 @ self defaultLineHeight) color: self defaultTextCursorColor].
			offset := tmp first].! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/15/2016 14:37'!
estimatedLinesHeight
	"Without considering precise character breaks, calculate the expected height of all lines."

	| cpl |
	self model lines ifEmpty: [^ 0].
	
	cpl := self width // (self defaultFont widthOf: $x).
	
	^ (self model lines gather: [:lineWithBreaks |
			lineWithBreaks contents
				ifEmpty: [{1}]
				ifNotEmpty: [:string | string lines collect: [:line | (line size / cpl) ceiling max: 1 "empty lines"]]]) sum
		
		* self defaultLineHeight! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/15/2016 17:13'!
estimatedPromptHeight

	| cpl prompt |
	self promptVisible ifFalse: [^ 0].
	
	cpl := self width // (self defaultFont widthOf: $x).
	prompt := self model prompt.
	
	^ ((prompt lines collect: [:line | (line size / cpl) ceiling max: 1 "empty lines"]) sum + ((prompt endsWith: String cr) ifTrue: [1] ifFalse: [0]))
		* self defaultLineHeight! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/14/2016 11:22'!
extent

	^ Display extent! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/14/2016 12:10'!
height

	^ self extent y! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/14/2016 11:35'!
minimumWidth

	^ (self defaultFont widthOf: $x) * 5! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 10:53'!
model

	^ model! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 11:10'!
model: anObject

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].

	model := anObject! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 2/14/2016 16:44'!
offsetForAutoScroll

	^ 0 @ ((self height - (self estimatedLinesHeight + self estimatedPromptHeight)) min: 0)! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:08'!
promptVisible

	^ promptVisible! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/15/2016 17:08'!
promptVisible: boolean

	promptVisible := boolean.! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 2/14/2016 16:40'!
scrollDown: lines

	self autoScroll ifTrue: [^ self].

	self offsetForAutoScroll in: [:o |
		self scrollingOffset: self scrollingOffset - (0@ (self defaultLineHeight * lines)).		
		self scrollingOffset <= o ifTrue: [self autoScroll: true]].
! !

!SqshView methodsFor: 'scrolling' stamp: 'mt 2/14/2016 16:31'!
scrollUp: lines

	self autoScroll ifTrue: [self scrollingOffset: self offsetForAutoScroll].

	self autoScroll: false.
	self scrollingOffset: ((self scrollingOffset + (0@ (self defaultLineHeight * lines))) min: 0@0).! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:16'!
scrollingOffset
	^ scrollingOffset! !

!SqshView methodsFor: 'accessing' stamp: 'mt 2/14/2016 16:35'!
scrollingOffset: point

	scrollingOffset := point.
	self updateDisplay.! !

!SqshView methodsFor: 'display' stamp: 'mt 2/14/2016 14:45'!
update: aspect

	(#(cursor input lines) includes: aspect)
		ifTrue: [self updateDisplay].! !

!SqshView methodsFor: 'display' stamp: 'mt 2/14/2016 14:44'!
updateDisplay

	needsDisplayUpdate := true.! !

!SqshView methodsFor: 'geometry' stamp: 'mt 2/14/2016 12:10'!
width

	^ self extent x! !

Object subclass: #SqueakShell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakShell'!

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:15'!
logo

	^ '
       ###                                                          ####        
     ########                                                    ####  ###      
    ##      ###                                                ###      ###     
   ###       ###                                              ###        ##     
   ##          ###                                           ##          ##     
   ##           ###                                        ###            ##    
   ##            ###                                      ###             ##    
   ##             ###                                    ###              ##    
   ##              ###                                  ###               ##    
   ##               ###                                ###               ##     
   ##                ###                              ###                ##     
   ##                 ###                            ###                 ##     
   ##                  ###                           ##                  ##     
    ##                  ##                           #                   ##     
    ##                                                                  ##      
    ##                                                                  ##      
     ##                                                                 ##      
     ##                                                                ##       
     ###                 ########              #####                   ##       
      ##                ##########            ########                ###       
      ##               #######  ##           ##########               ##        
       ##              #######  ###         ####### ###              ###        
       ##              #######  ###         ######   ###             ##         
                       ###########          ####### ####                        
                        ##########          ###########                         
                         ########            #########                          
                           ####               #######                           
                                                                                
                                                                                
     #####################           ######                                     
##########            #########   ###########     ###########################   
##                           ##  #############  #######                 ########
            ##################   ##############                                 
      #############   ########   ##############  #####################          
  ######                          ############                     #######      
####             ############      #########                             #####  
#           ##################                   ###############            ####
          ####                                   #            ######           #
        ###                                                       ####          
      ###                                                           ####        
                                                                      ##        '! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:16'!
open

	| world |
	world := SqshControllerManager new.
	[world doOneCycle. Processor yield] repeat.! !

!SqueakShell class methodsFor: 'as yet unclassified' stamp: 'mt 2/15/2016 18:16'!
openAsProject

	SqshProject new enter.! !

!Debugger class methodsFor: '*SqueakShell-instance creation' stamp: 'mt 2/15/2016 15:44'!
sqshContext: aContext
	"Answer an instance of me for debugging the active process starting with the given context."

	^ self new
		process: Processor activeProcess
		controller: nil
		context: aContext! !

!Debugger class methodsFor: '*SqueakShell-opening' stamp: 'mt 2/15/2016 18:25'!
sqshOpenContext: aContext label: aString contents: contentsStringOrNil

	| process errorWasInUIProcess |
	process := Processor activeProcess.
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: process.

	ErrorRecursion not & Preferences logDebuggerStackToFile ifTrue:
		[Smalltalk logSqueakError: aString inContext: aContext].
	ErrorRecursion ifTrue:[
		ErrorRecursion := false.
		self primitiveError: aString].
	ErrorRecursion := true.

	Project current addDeferredUIMessage: [
			Project current world activeController
				showMessage: aString;
				showMessage: contentsStringOrNil;
				showStack: aContext.
			process terminate. "No support for debugging yet."
	].

	ErrorRecursion := false.

	process suspend.! !

!Debugger class methodsFor: '*SqueakShell-opening' stamp: 'mt 2/15/2016 18:25'!
sqshOpenInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."

	| errorWasInUIProcess |
	self assert: interruptedProcess isSuspended.
	
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: interruptedProcess.

	Preferences logDebuggerStackToFile ifTrue:
		[(aString includesSubstring: 'Space') & (aString includesSubstring: 'low')
			ifTrue: [Smalltalk logError: aString inContext: interruptedProcess suspendedContext to: 'LowSpaceDebug.log']
			ifFalse: [Smalltalk logSqueakError: aString inContext: interruptedProcess suspendedContext]].
		
	Project current addDeferredUIMessage: [
			Project current world activeController in: [:controller |
				controller showMessage: aString.
				controller showStack: interruptedProcess suspendedContext].
			interruptedProcess terminate. "No support for debugging yet."
	].! !

!Debugger class methodsFor: '*SqueakShell-opening' stamp: 'mt 2/15/2016 18:24'!
sqshOpenOn: process context: context label: title contents: contentsStringOrNil fullView: bool

	| errorWasInUIProcess |
	errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: process.
	
	[
		Preferences logDebuggerStackToFile
			ifTrue: [Smalltalk logSqueakError: title inContext: context].			
	] on: Error do: [:ex | ex return: nil].

	Project current addDeferredUIMessage: [
		[
			Project current world activeController in: [:controller |
				controller showMessage: title.
				contentsStringOrNil ifNotNil: [:s | controller showMessage: s].
				controller showStack: context].
			process terminate. "No support for debugging yet."
			
		] on: Error do: [:ex |
					self primitiveError: 
						'Original error: ', 
						title asString, '.
		Debugger error: ', 
					([ex description] on: Error do: ['a ', ex class printString]), ':'
				].
	].

	process suspend.! !
